#
# advancedPage.py - GUI for advanced options page in
#                   system-config-securitylevel
#
# Chris Lumens  <clumens@redhat.com>
#
# Copyright 2007 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
class AdvancedPage:
    def __init__(self, xml):
        self.xml = xml
        self.origFilename = None
        self.dirty = False

        # Bring in widgets from glade file.
        self.advancedVBox = self.xml.get_widget("advancedVBox")
        self.customRulesCheckbox = self.xml.get_widget("customRulesCheckbox")
        self.customRulesChooser = self.xml.get_widget("customRulesChooser")

        # Set up initial sensitivity of things on the advanced page.
        self.customRulesChooser.set_sensitive(False)
        self.customRulesCheckbox.connect("clicked", self.custom_changed_cb)

    def custom_changed_cb(self, button, *args):
        self.customRulesChooser.set_sensitive(button.get_active())

    def set_sensitive(self, bool):
        self.customRulesCheckbox.set_sensitive(bool)

        if self.customRulesCheckbox.get_active():
            self.customRulesChooser.set_sensitive(bool)

    def set_filename(self, file):
        self.customRulesChooser.set_filename(file)

    def apply(self):
        if self.customRulesCheckbox.get_active():
            newFilename = self.customRulesChooser.get_filename()

            if newFilename != self.origFilename:
                self.dirty = True

            if newFilename != "":
                return "--custom-rules=%s" % newFilename
            else:
                return ""
        else:
            # They used to have a custom rules file enabled, but no longer do.
            # Set the dirty flag so we know to re-run lokkit without the arg
            if self.origFilename:
                self.dirty = True

            return ""
