#
# securitylevel.py - GUI front end code for basic system security
#
# Brent Fox <bfox@redhat.com>
#
# Copyright 2002, 2003, 2004 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import string
import gtk
import gtk.glade
import gobject
import sys
import os
import socket
import re
sys.path.append('/usr/share/system-config-securitylevel')
import scs_checklist as checklist
import advancedPage
import selinuxPage

##
## I18N
## 
from rhpl.translate import _, N_
import rhpl.translate as translate
domain = "system-config-securitylevel"
translate.textdomain (domain)

##
## Icon for windows
##

iconPixbuf = None      
try:
    iconPixbuf = gtk.gdk.pixbuf_new_from_file("/usr/share/system-config-securitylevel/pixmaps/system-config-securitylevel.png")
except:
    pass

##
## Pull in the Glade file
##
if os.access("system-config-securitylevel.glade", os.F_OK):
    xml = gtk.glade.XML ("system-config-securitylevel.glade", domain=domain)
else:
    xml = gtk.glade.XML ("/usr/share/system-config-securitylevel/system-config-securitylevel.glade", domain=domain)

class Service:
    def __init__ (self, name="", description="", ports=[], modules=[]):
        self.name = name
        self.description = description
        self.ports = ports
        self.modules = modules

class childWindow:
    # You must specify a runPriority for the firstboot module order
    runPriority = 50
    moduleName = _("Firewall")
    commentTag = _("Configure firewall rules and SELinux setting")
    shortMessage = _("You can use a firewall to allow access to specific "
                     "services on your computer from other computers and "
                     "prevent unauthorized access from the outside world.  "
                     "Which services, if any, do you wish to allow access to?")

    def destroy(self, args):
        gtk.main_quit()

    def __init__(self):
        self.xml = xml
        self.dirty = False
        self.doDebug = None
        self.selinuxPage = None
        self.advancedPage = None
        self.other_changed_firsttime = True

        self.serviceList = [Service(name="ssh", description="SSH",
                                    ports=[("ssh", "tcp")]),
                            Service(name="telnet", description="Telnet",
                                    ports=[("telnet", "tcp")]),
                            Service(name="http", description="WWW (HTTP)",
                                    ports=[("http", "tcp")]),
                            Service(name="ftp", description="FTP",
                                    ports=[("ftp", "tcp")],
                                    modules=["ip_conntrack_ftp"]),
                            Service(name="nfs", description="NFS4",
                                    ports=[("nfs", "tcp")]),
                            Service(name="https", description=_("Secure WWW (HTTPS)"),
                                    ports=[("https", "tcp")]),
                            Service(name="smtp", description=_("Mail (SMTP)"),
                                    ports=[("smtp", "tcp")]),
                            Service(name="samba", description=_("Samba"),
                                    ports=[("netbios-ns", "udp"),
                                           ("netbios-dgm", "udp"),
                                           ("netbios-ssn", "tcp"),
                                           ("microsoft-ds", "tcp")])
                           ]

    def setupScreen(self):
        # Bring in widgets from glade file.
        self.mainWindow = self.xml.get_widget("mainWindow")

        self.notebook = self.xml.get_widget("scsNotebook")
        self.mainVBox = self.xml.get_widget("mainVBox")
        self.advancedVBox = self.xml.get_widget("advancedVBox")
        self.seLinuxVBox = self.xml.get_widget("seLinuxVBox")

        self.incomingSW = self.xml.get_widget("incomingSW")
        self.trustedServicesBox = self.xml.get_widget("trustedServicesBox")
        self.otherPortsView = self.xml.get_widget("otherPortsView")
        self.otherPortsExpander = self.xml.get_widget("otherPortsExpander")
        self.addPortButton = self.xml.get_widget("addPortButton")
        self.removePortButton = self.xml.get_widget("removePortButton")

        self.securityOptionMenu = self.xml.get_widget("securityOptionMenu")

        self.addPortDialog = self.xml.get_widget("addPortDialog")
        self.addPortTable = self.xml.get_widget("addPortTable")
        self.portEntry = self.xml.get_widget("portEntry")

        self.mainWindow.set_icon(iconPixbuf)

        # Set up the enabled/disabled firewall combo box.
        listStore = gtk.ListStore(gobject.TYPE_STRING)
        self.securityOptionMenu.set_model(listStore)
        cell = gtk.CellRendererText()
        self.securityOptionMenu.pack_start(cell, True)
        self.securityOptionMenu.add_attribute(cell, 'text', 0)

        self.security_changed_firsttime = True
        self.securityOptionMenu.connect('changed', self.security_changed_cb)
        self.securityOptionMenu.append_text(_("Enabled"))
        self.securityOptionMenu.append_text(_("Disabled"))

        # Set up the trusted services checklist.
        self.incomingList = checklist.CheckList(columns=1)
        keyList = map(lambda svc: svc.description, self.serviceList)
        keyList.sort()

        for item in keyList:
            self.incomingList.append_row((item, ""), False)
        self.incomingSW.add(self.incomingList)        

        # Set up the view and columns for the Other Ports section.
        self.otherPortsStore = gtk.ListStore(gobject.TYPE_STRING,
                                             gobject.TYPE_STRING)
        otherPortsSorted = gtk.TreeModelSort(self.otherPortsStore)
        otherPortsSorted.set_sort_column_id(0, gtk.SORT_ASCENDING)
        self.otherPortsView.set_model(otherPortsSorted)

        portsCol = gtk.TreeViewColumn("Ports", gtk.CellRendererText(), text=0)
        portsCol.set_expand(True)
        self.otherPortsView.append_column(portsCol)
        protoCol = gtk.TreeViewColumn("Proto", gtk.CellRendererText(), text=1)
        self.otherPortsView.append_column(protoCol)

        self.addPortButton.connect("clicked", self.add_port_cb)
        self.removePortButton.connect("clicked", self.remove_port_cb)

        # Add buttons to the "Add Port" dialog, since they don't want to
        # behave otherwise.
        self.addPortDialog.add_buttons(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                       gtk.STOCK_OK, gtk.RESPONSE_OK)

        # Add options to the protocol combo box.
        self.protoCombo = gtk.combo_box_new_text()
        self.addPortTable.attach(self.protoCombo, 1, 2, 1, 2)
        self.protoCombo.append_text("tcp")
        self.protoCombo.append_text("udp")
        self.protoCombo.set_active(0)

        # Put labels on the notebook tabs since gazpacho doesn't provide a
        # way to do that right now.
        firewallLabel = gtk.Label(_("_Firewall Options"))
        firewallLabel.set_use_underline(True)
        self.notebook.set_tab_label(self.mainVBox, firewallLabel)
        selinuxLabel = gtk.Label(_("_SELinux"))
        selinuxLabel.set_use_underline(True)
        self.notebook.set_tab_label(self.seLinuxVBox, selinuxLabel)
        advancedLabel = gtk.Label(_("Ad_vanced Options"))
        advancedLabel.set_use_underline(True)
        self.notebook.set_tab_label(self.advancedVBox, advancedLabel)

    def firewall_activated(self, *args):
        self.trustedServicesBox.set_sensitive(True)
        self.otherPortsExpander.set_sensitive(True)

        if self.advancedPage:
            self.advancedPage.set_sensitive(True)

        self.dirty = True

    def none_activated(self, *args):
        self.trustedServicesBox.set_sensitive(False)
        self.otherPortsExpander.set_sensitive(False)

        if self.advancedPage:
            self.advancedPage.set_sensitive(False)

        self.dirty = True

    def add_port_cb(self, button, *args):
        protoMapping = ["tcp", "udp"]

        self.addPortDialog.show_all()
        self.portEntry.set_text("")
        self.protoCombo.set_active(0)

        # Loop until we get a valid port string.
        while True:
            result = self.addPortDialog.run()

            if result == gtk.RESPONSE_OK:
                portStr = self.portEntry.get_text()
                proto = protoMapping[self.protoCombo.get_active()]

                if not self.portIsValid(portStr):
                    self.invalidPortDialog(portStr)
                    continue
                else:
                    if self._findPortInStore(portStr, proto) is None:
                        self.otherPortsStore.append([portStr, proto])
                        self.dirty = True
                    break
            else:
                break

        self.addPortDialog.hide()

    def remove_port_cb(self, button, *args):
        selection = self.otherPortsView.get_selection()
        (model, treeModelSortIter) = selection.get_selected()

        if treeModelSortIter is None:
            return

        # Convert the iter on the TreeModelSort to an iter on the underlying
        # ListStore so we can delete the right thing.
        iter = model.convert_iter_to_child_iter(None, treeModelSortIter)

        if iter is not None:
            # Need to get past the TreeModelSort to whatever's underneath.
            model.get_model().remove(iter)
            self.dirty = True

    def security_changed_cb(self, combobox, *args):
        if self.security_changed_firsttime:
            self.security_changed_firsttime = False
            return
        active = combobox.get_active()
        if active < 0:
            return None
        elif active == 0:
            self.firewall_activated()
        elif active == 1:
            self.none_activated()
        else:
            return None

    def other_changed_cb(self, *args):
        if self.other_changed_firsttime:
            self.other_changed_firsttime = False
            return
        self.dirty = True

    def okClicked(self, *args):
        rc = self.apply()
        if rc == 0:
            self.destroy(args)

    def applyClicked(self, *args):
        self.apply()

    def _getServiceByDesc(self, desc):
        for x in self.serviceList:
            if x.description == desc:
                return x

        return None

    def _findPortInStore(self, port, proto):
        iter = self.otherPortsStore.get_iter_first()
        while iter:
            if port == self.otherPortsStore.get_value(iter, 0) and \
               proto == self.otherPortsStore.get_value(iter, 1):
                return iter
            else:
                iter = self.otherPortsStore.iter_next(iter)

        return None

    def addServiceToSet(self, service, proto, serviceSet, portSet):
        if service in ["21", "ftp"]:
            serviceSet.add("ftp")
        elif service in ["22", "ssh"]:
            serviceSet.add("ssh")
        elif service in ["23", "telnet"]:
            serviceSet.add("telnet")
        elif service in ["25", "smtp"]:
            serviceSet.add("smtp")
        elif service in ["80", "http"]:
            serviceSet.add("http")
        elif service in ["443", "https"]:
            serviceSet.add("https")
        elif service in ["2049", "nfs"]:
            serviceSet.add("nfs")
        else:
            # Catch ports that aren't in /etc/services.
            try:
                protoname = socket.getservbyport(int(service), proto)
            except:
                protoname = service

            portSet.add(protoname + ":" + proto)

    def parseArgList(self, args):
        devicesList = []
        masqList = []
        serviceSet = set()
        portSet = set()

        # Define a set so we can quickly test if all the samba ports are
        # enabled or not.
        sambaSet = set()
        sambaSet.add("netbios-ns:udp")
        sambaSet.add("netbios-dgm:udp")
        sambaSet.add("netbios-ssn:tcp")
        sambaSet.add("microsoft-ds:tcp")

        self.trustedServicesBox.set_sensitive(True)
        self.otherPortsExpander.set_sensitive(True)

        for arg in args:
            if arg != "" and arg[0] != "#":
                if arg in ["--high", "--medium", "--enabled"]:
                    self.securityOptionMenu.set_active(0)
                    self.firewall_activated()
                    self.dirty = False
                elif arg == "--disabled":
                    self.securityOptionMenu.set_active(1)
                    self.none_activated()
                    self.dirty = False
                elif arg.startswith("--trust="):
                    key, device = string.split(arg, "=")
                    devicesList.append(string.strip(device))
                elif arg.startswith("--masq="):
                    key, device = string.split(arg, "=")
                    masqList.append(string.strip(device))
                elif arg.startswith("--port="):
                    key, value = string.split(arg, "=")

                    try:
                        service, protocol = string.split(value, ":")
                    except ValueError:
                        service = value
                        protocol = "tcp"

                    service = string.strip(service)
                    protocol = string.strip(protocol)

                    self.addServiceToSet(service, protocol, serviceSet, portSet)
                elif arg.startswith("--custom-rules="):
                    key, file = string.split(arg, "=")

                    if self.advancedPage:
                        self.advancedPage.customRulesCheckbox.set_active(True)
                        self.advancedPage.set_sensitive(True)
                        self.advancedPage.origFilename = file
                        self.advancedPage.set_filename(file)

        if sambaSet.issubset(portSet):
            # Remove samba ports, since we don't want them in both places.
            portSet = portSet.difference(sambaSet)

            iter = self.incomingList.store.get_iter_first()
            while iter:
                if self.incomingList.store.get_value(iter, 1) == _("Samba"):
                    self.incomingList.store.set_value(iter, 0, True)
                    break

                iter = self.incomingList.store.iter_next(iter)

        # Check the enabled supported services.  We do this backwards by
        # iterating over all the services listed in the UI and checking them if
        # the corresponding ports are listed in what was provided by the user.
        iter = self.incomingList.store.get_iter_first()
        while iter:
            val = self.incomingList.store.get_value(iter, 1)
            svc = self._getServiceByDesc(val)

            # If all the service's ports are listed, check it.
            for (port, proto) in svc.ports:
                if port in serviceSet:
                    self.incomingList.store.set_value(iter, 0, True)

            iter = self.incomingList.store.iter_next(iter)

        # Add the enabled other ports to the view.
        for x in portSet:
            (port, proto) = x.split(":")
            if self.portIsValid(port) and proto in ["tcp", "udp"]:
                self.otherPortsStore.append([port, proto])

    def readFile(self):
        path = "/etc/sysconfig/system-config-securitylevel"
        if os.access(path, os.R_OK) == 1:
            lines = open(path, 'r').readlines()
        else:
            #The file isn't there, so just return and keep on going
            self.trustedServicesBox.set_sensitive(True)
            self.otherPortsExpander.set_sensitive(True)
            return

        self.parseArgList(map(lambda x: x.rstrip(), lines))

    def portIsValid(self, port):
        namepat = re.compile("[^-0-9]")
        pattern = re.compile("^[0-9]+(-[0-9]+)?$")
        return pattern.match(port) is not None or \
               namepat.match(port) is not None

    def invalidPortDialog(self, port):
        text = _("Invalid port given: '%s'.  Please give a port number "
                 "or service name.") % port

        dlg = gtk.MessageDialog(None, 0, gtk.MESSAGE_WARNING, gtk.BUTTONS_OK,
                                text)
        dlg.set_modal(True)
        dlg.set_icon(iconPixbuf)
        dlg.set_position(gtk.WIN_POS_CENTER)
        dlg.show_all()
        dlg.run()
        dlg.destroy()

    def confirmDialog(self):
        dlg = gtk.MessageDialog(None, 0, gtk.MESSAGE_WARNING, gtk.BUTTONS_YES_NO,
                                _("Clicking the 'Yes' button will set the security level of the "
                                "system and override any "
                                "existing firewall configuration.  Are you sure that you want "
                                "to do this?"))
        dlg.set_position(gtk.WIN_POS_CENTER)
        dlg.set_modal(True)
        dlg.set_icon(iconPixbuf)
        dlg.show_all()
        result = dlg.run()
        dlg.destroy()
        return result

    def apply(self, *args):
        # With the new enabled/disabled behavior, we have to ignore the config
        # file or else you can only ever turn on services.
        args = ['--quiet', '-f']

        if self.advancedPage:
            advancedArgs = self.advancedPage.apply()
            self.dirty = self.advancedPage.dirty
            args.append(self.advancedPage.apply())

        if self.selinuxPage and self.selinuxPage.selinuxsupport:
            self.selinuxPage.apply()

        if not self.dirty and not self.incomingList.dirty:
            return 0

        index = self.securityOptionMenu.get_active()
        if index == 0:
            args.append('--enabled')
        elif index == 1:
            args.append('--disabled')

        iter = self.incomingList.store.get_iter_first()
        while iter:
            svc = self._getServiceByDesc(self.incomingList.store.get_value(iter, 1))
            if self.incomingList.store.get_value(iter, 0):
                for (port, proto) in svc.ports:
                    args.append("--port=%s:%s" % (port, proto))
                for module in svc.modules:
                    args.append("--addmodule=%s" % module)
            else:
                for module in svc.modules:
                    args.append("--removemodule=%s" % module)

            iter = self.incomingList.store.iter_next(iter)

        model = self.otherPortsView.get_model()
        iter = model.get_iter_first()

        while iter:
            args.append("--port=%s:%s" % (model.get_value(iter, 0),
                                          model.get_value(iter, 1)))
            iter = model.iter_next(iter)

        if self.confirmDialog() == gtk.RESPONSE_NO:
            return None

        if self.doDebug:
            print "don't call lokkit if in debug mode"
        else:
            path = '/usr/sbin/lokkit'

            lokkit = os.fork()
            if not lokkit:
                args.insert(0, path)
                os.execv(path, args)                

        # Set these to False so if you click Apply followed by OK, you're not
        # prompted twice.
        self.dirty = False
        self.incomingList.dirty = False
        return 0

    def launch(self, doDebug = None):
        self.doDebug = doDebug
        self.setupScreen()

        messageLabel = gtk.Label(_(self.shortMessage))
        messageLabel.set_line_wrap(True)
        messageLabel.set_size_request(500, -1)
        messageLabel.set_alignment(0.0, 0.5)

        self.readFile()
        vbox = gtk.VBox(spacing=10)
        vbox.pack_start(messageLabel, expand=False)
        self.mainVBox.reparent(vbox)

        icon = gtk.Image()
        icon.set_from_pixbuf(iconPixbuf)
        return vbox, icon, self.moduleName

    def stand_alone(self):
        desktopName = _("Firewall and SELinux")

        self.setupScreen()
        self.okButton = self.xml.get_widget("okButton")
        self.cancelButton = self.xml.get_widget("cancelButton")
        self.applyButton = self.xml.get_widget("applyButton")
        self.advancedPage = advancedPage.AdvancedPage(xml)
        self.selinuxPage = selinuxPage.selinuxPage(xml)

        self.mainWindow.connect("destroy", self.destroy)
        self.okButton.connect("clicked", self.okClicked)
        self.cancelButton.connect("clicked", self.destroy)
        self.applyButton.connect("clicked", self.applyClicked)

        self.readFile()
        self.mainWindow.show_all()
        gtk.main()
